﻿
using System;
using System.Drawing;
using System.Windows.Forms;

/* Integration by the use of a Legendre or Chebychev polynomials     */
/*                          www.mosismath.com                        */


namespace WindowsFormsApplication1
{

    public partial class MainWin : Form
    {
        public const int order = 10; // number of polynomial values (degree + 1)
        public const double startingpoint = 1.0; // number of reference points
        public const int datapoints = 1000; // number of data points for the interpolation
        public const double deltaX = 0.5; // sample time 
       
        double[] yp = new double[datapoints];
        double[] xp = new double[datapoints];
        double[] y_k = new double[order];
        double[] x = new double[order];
        bool[] valid = new bool[order];
        double[] l = new double[order];

        double a = 0;             // range for the integration
        double b = Math.PI / 2;

        private double f_x(double x)
        {
            return 5.0 / (Math.Exp(Math.PI) - 2.0) * Math.Exp(2 * x) * Math.Cos(x);
        }

     

        private double[] NextLegendre(double[] pn2, double[] pn1, int n)
        {
            int i;
            double[] tempPoly = new double[pn2.Length + 1];
            for (i = 0; i < pn2.Length; i++)
            {
                tempPoly[i] = pn2[i] * (2 * n + 1) / (n + 1);
            }
            for (i = 0; i < pn1.Length; i++)
            {
                tempPoly[i + 2] = tempPoly[i + 2] - pn1[i] * n / (n + 1);
            }
            return tempPoly;
        }


        public double[] CalcLegendre(int n)
        {
            double[] p2 = new double[3]; // P2 = 1/2*(3x^2 - 1)
            double[] p1 = new double[2]; // P1 = x 
            double[] poly = new double[n];
            int i;

            p2[2] = -0.5;
            p2[1] = 0;
            p2[0] = 1.5;

            p1[1] = 0;
            p1[0] = 1.0;

            for (i = 2; i < n; i++)  // loop from p2 to pn
            {
                poly = NextLegendre(p2, p1, i);
                p1 = p2;
                p2 = poly;
            }
            return poly;
        }


        private double[] NextChebychev(double[] pn2, double[] pn1, int n)
        {
            int i;
            double[] tempPoly = new double[pn2.Length + 1];
            for (i = 0; i < pn2.Length; i++)
            {
                tempPoly[i] = pn2[i] * 2;
            }
            for (i = 0; i < pn1.Length; i++)
            {
                tempPoly[i + 2] = tempPoly[i + 2] - pn1[i];
            }
            return tempPoly;
        }


        public double[] CalcLChebychef(int n)
        {
            double[] p2 = new double[3]; // P2 = 2x^2 - 1
            double[] p1 = new double[2]; // P1 = x + 0
            double[] poly = new double[n];
            int i;

            p2[2] = -1;
            p2[1] = 0;
            p2[0] = 2;

            p1[1] = 0;
            p1[0] = 1;

            for (i = 2; i < n; i++)  // loop from p2 to pn
            {
                poly = NextChebychev(p2, p1, i);
                p1 = p2;
                p2 = poly;
            }
            return poly;
        }

        private void FindRoots() // Newton-Maehly algorythm
        {
            double bn;
            double cn;
            double xk, xk_old, dx, dx_old;
            double xz;
            int i, j, n;
            n = 0;
            xk_old = startingpoint;
            xk = xk_old;
            while (n < order)
            {
                if (n == 0)
                {
                    xk = xk_old;
                }
                else
                {
                    xk = (xk + xk_old) / 2.0;
                    xk_old = xk;
                }
                xz = 0;
                dx = xk;
                dx_old = 1E8;
                while ((Math.Abs(dx) > 1E-8) && (Math.Abs(xk) < 1E30)) // check if we are close enough
                {
                    dx_old = dx;
                    bn = l[0];
                    cn = bn;
                    for (i = 1; i < order+1; i++)
                    {
                        bn = xk * bn + l[i];
                        if (i < order)
                            cn = xk * cn + bn;
                    }
                    if (n == 0)
                    {
                        if (cn != 0) // no root found jet => go the the next P
                        {
                            dx = bn / cn;
                            xk = xk - dx;
                        }
                        else
                            if (Math.Abs(bn) < 1E-20) // that's allready a root point
                                dx = 0.0;
                    }
                    else
                    {
                        if (Math.Abs(bn) > 1E-18)  // one root found => keep it and subtract it from the polynomial
                        {
                            xz = 0;
                            for (j = 0; j < n; j++)
                                xz = xz + 1.0 / (xk - x[j]);
                            dx = 1.0 / (cn / bn - xz);
                            xk = xk - dx;
                        }
                        else
                            xk = xk + 0.5; // we started right at one root point => just move a bit :-)
                    }
                }
                if (Math.Abs(xk) < 1E20)
                {
                    if (Math.Abs(dx) <= Math.Abs(dx_old))  // register root in the list
                    {
                        x[n] = xk;
                        valid[n] = (Math.Abs(dx) != 0) || (n == 0); //(a[order - 1] != 0);
                    }
                    else
                    {
                        x[n] = 0;
                        valid[n] = false;
                    }
                    n++;
                }
                else
                    n = order;
            }
        }


        public double[] multPoly(double[] p1, double[] p2)
        {
            int i, j;
            double[] tempPoly = new double[p1.Length + p2.Length - 1];
            for (i = 0; i < p1.Length; i++)
            {
                for (j = 0; j < p2.Length; j++)
                {
                    tempPoly[j + i] = tempPoly[j + i] + p2[j] * p1[i];
                }
            }
            return tempPoly;
        }


        public void Interpolate(int maxPoints, double minX, double maxX)
        {
          int i, k;
            for (i = 0; i < maxPoints; i++)
            {
                yp[i] = 0;
                xp[i] = ((double)(i) * (maxX - minX) / (double)maxPoints + minX);
                for (k = 0; k <= order; k++)
                {
                    yp[i] = yp[i] + l[k] * Math.Pow(xp[i], (double)(order - k));
                }
            }
        }


        private double CalcWeights(int index)
        {
            int i;
            double a = -1;
            double b = 1;
            double res = 0;
            double[] wl = new double[1];
            double[] poly = new double[2];
            wl[0] = 1;
            // build polynomial
            for (i = 0; i < x.Length; i++)
            {
                if (i != index)
                {
                    poly[0] = -x[i] / (x[index] - x[i]);
                    poly[1] = 1 / (x[index] - x[i]);
                    wl = multPoly(wl, poly);
                }
            }
            // build integral
            res = wl[0] * (b - a);
            for (i = 1; i < x.Length; i++)
            {
                res = res + wl[i] * (Math.Pow(b, i + 1) - Math.Pow(a, i + 1)) / (i + 1);
            }
            return res;
        }


        private void Integrate()
        {
            int i;
            double integral = 0;
            double[] w = new double[x.Length];
            i = 0;
            for (i = 0; i < order; i++)
            {
                y_k[i] = f_x(a + (b - a) * (1 - x[i]) / 2);
            }

            integral = 0;
            for (i = 0; i < order; i++)
            {
                integral = integral + y_k[i] * CalcWeights(i);
            }
            integral = integral / 2 * (b - a);
            tbResult.Text = Convert.ToString(integral);
        }


        public void DrawGraph(int maxPoints, double minX, double maxX, bool doClear)
        {
            Point p1, p2;
            int j;
            double maxTime = -1.0;
            double minTime = 1.0;
            double maxVal = -1.0;
            double minVal = 1.0;
            double scalefactor;
            bool drawCeroline;
            for (j = 0; j < maxPoints - 1; j++)
            {
                if ((xp[j] >= minX) && (xp[j] <= maxX))
                {
                    if (maxTime < xp[j])
                        maxTime = xp[j];
                    if (minTime > xp[j])
                        minTime = xp[j];
                    if (maxVal < yp[j])
                        maxVal = yp[j];
                    if (minVal > yp[j])
                        minVal = yp[j];
                }
            }
            maxTime = maxTime * 1.05;
            maxTime = maxTime * 1.05;
            if (minVal < 0)
            {
                if (maxVal > 0)
                {
                    if (maxVal > Math.Abs(minVal))
                        scalefactor = pGraph.Height / maxVal / 2.2;
                    else
                        scalefactor = pGraph.Height / Math.Abs(minVal) / 2.2;
                }
                else
                    scalefactor = 5.0;
                drawCeroline = true;
            }
            else
            {
                if (maxVal > 0)
                    scalefactor = pGraph.Height / (maxVal) / 1.1;
                else
                    scalefactor = 1.0;
                drawCeroline = false;
            }
            p1 = new Point();
            p2 = new Point();
            Graphics g = pGraph.CreateGraphics();
            if (doClear)
                g.Clear(Color.White);
            Pen bluePen = new Pen(Color.Blue, 2);
            Pen redPen = new Pen(Color.Red, 2);
            Pen blackPen = new Pen(Color.Black, 2);
            if (drawCeroline)
            {
                p1.X = 0;
                p1.Y = (pGraph.Height / 2);
                p2.X = pGraph.Width;
                p2.Y = (pGraph.Height / 2);
                g.DrawLine(blackPen, p1, p2);
            }
            bluePen.Width = 1;
            if (maxTime > 0) // draw interpolated graph
            {
                for (j = 0; j < maxPoints - 1; j++)
                {
                    if ((xp[j] >= minX) && (xp[j] <= maxX))
                    {
                        p1.X = Convert.ToInt32(((xp[j] - (double)(minTime)) / (double)(maxTime - minTime)) * pGraph.Width);
                        p2.X = Convert.ToInt32(((xp[j + 1] - (double)(minTime)) / (double)(maxTime - minTime)) * pGraph.Width);
                        if (drawCeroline)
                        {
                            p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                            p2.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                        }
                        else
                        {
                            p1.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                            p2.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                        }
                        if (p2.X > p1.X)
                            g.DrawLine(bluePen, p1, p2);
                    }
                }
                for (j = 0; j < order; j++) // mark reference points
                {
                    float emSize = 10;
                    if (valid[j])
                    {
                        p1.X = Convert.ToInt32((x[j] - (double)(minTime)) * pGraph.Width / (double)(maxTime - minTime));
                        if (drawCeroline)
                            p1.Y = (pGraph.Height / 2); 
                        else
                            p1.Y = (pGraph.Height);
                        g.DrawEllipse(redPen, (int)(p1.X - 1.0), (int)(p1.Y - 1.0), 2, 2);
                        g.DrawString(Convert.ToString(Math.Round(x[j], 2)), new Font(FontFamily.GenericSansSerif, emSize, FontStyle.Regular),
                                      new SolidBrush(Color.Black), p1.X, p1.Y + 10);
                    }
                }
            }
            g.Dispose();
        }


        public MainWin()
        {
            InitializeComponent();
        }


        private void pGraph_Paint(object sender, PaintEventArgs e)
        {
            DrawGraph(datapoints, -1, 1, false);
        }


        private void btChebychev_Click(object sender, EventArgs e)
        {
            l = CalcLChebychef(order);
            FindRoots();
            Interpolate(datapoints, -1.5, 1.5);
            DrawGraph(datapoints, -1, 1, true);
            Integrate();
        }


        private void btLegendre_Click(object sender, EventArgs e)
        {
            l = CalcLegendre(order);
            FindRoots();
            Interpolate(datapoints, -1.5, 1.5);
            DrawGraph(datapoints, -1, 1, true);
            Integrate();
        }  
    }
}
